﻿using UnityEngine;
using System;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using Random = UnityEngine.Random;
using MeshVR;
using System.Linq;

namespace PluginIdea.AME
{
    public class AWindZone : MVRScript
    {

        private Transform rescaleObject;
        private WindZone windZone;

        private List<string> modeStrs = new List<string> { "Directional", "Spherical" };

        private JSONStorableStringChooser modeChooser;
        private JSONStorableFloat radius;
        private JSONStorableFloat windMain;
        private JSONStorableFloat windPulseMagnitude;
        private JSONStorableFloat windTurbulence;

        public override void Init()
        {
            rescaleObject = containingAtom.reParentObject.Find("object/rescaleObject");

            windZone = Utils.GetSafeComponent<WindZone>(rescaleObject.gameObject);

            UIInit();
        }

        private void UIInit()
        {
            Location.InitTData();

            modeChooser = Utils.CreateStringChooser(this, Location.GetStr(Location.WindZoneMode), modeStrs, false, true);
            modeChooser.setCallbackFunction = (val) => {
                bool isDirection = val == "Directional";
                windZone.mode = isDirection ? WindZoneMode.Directional : WindZoneMode.Spherical;
                Utils.ActiveSliderUI(radius.slider, !isDirection);
            };

            radius = Utils.CreateSliderFloat(this, Location.GetStr(Location.WindZoneRadius), 30, 1, 100, false, true);
            radius.setCallbackFunction = (val) => { windZone.radius = val; };

            windMain = Utils.CreateSliderFloat(this, Location.GetStr(Location.WindZoneMain), 0.2f, 0, 1, false, true);
            windMain.setCallbackFunction = (val) => { windZone.windMain = val; };

            windPulseMagnitude = Utils.CreateSliderFloat(this, Location.GetStr(Location.WindZonePulseMagnitude), 0.5f, 0, 3, false, true);
            windPulseMagnitude.setCallbackFunction = (val) => { windZone.windPulseMagnitude = val; };

            windTurbulence = Utils.CreateSliderFloat(this, Location.GetStr(Location.WindZoneTurbulence), 0.01f, 0, 1, false, true);
            windTurbulence.setCallbackFunction = (val) => { windZone.windTurbulence = val; };

            // Recall
            modeChooser.setCallbackFunction(modeChooser.val);
            radius.setCallbackFunction(radius.val);
            windMain.setCallbackFunction(windMain.val);
            windPulseMagnitude.setCallbackFunction(windPulseMagnitude.val);
            windTurbulence.setCallbackFunction(windTurbulence.val);
        }

        private void OnDestory()
        {

        }
    }
}
